from stapy.sta.entities.location import Location
from stapy.sta.entities.featureofinterest import FeatureOfInterest
from stapy.sta.entities.observation import Observation
from stapy.sta.entities.observedproperty import ObservedProperty
from stapy.sta.entities.datastream import Datastream
from stapy.sta.entities.sensor import Sensor
from stapy.sta.entities.thing import Thing
